var FlowCell = function()
{
	var flowcell = {};
	var debug = 0;
	var subtypePooledLibrary = null;
	var annotationTypePipeline = null;
	
	var flowCells = [];
	var selectedPools = [];
	var newPools;

	var PRESET_NEXTSEQ = 'NextSeq';
	var DEFAULT_READ_NEXTSEQ = [76, 6, 76];

	var poolsAreValid = false;
	var readIsValid = [];
	var currentFcNo;
	var currentLaneNo;

	// Page initialization
	flowcell.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', flowcell.validateStep1);
		Events.addEventHandler('preset', 'change', flowcell.presetOnChange);
		Events.addEventHandler('pools', 'change', flowcell.poolsOnChange);
		Events.addEventHandler('pools', 'base-selected', flowcell.setPoolCallback);
		Buttons.addClickHandler('btnSelectPools', flowcell.selectPools);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', flowcell.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', flowcell.validateStep2);
		Events.addEventHandler('read1', 'change', flowcell.readOnChange);
		Events.addEventHandler('indexRead', 'change', flowcell.readOnChange);
		Events.addEventHandler('read2', 'change', flowcell.readOnChange);
		Events.addEventHandler('read1', 'keypress', Events.integerOnly);
		Events.addEventHandler('indexRead', 'keypress', Events.integerOnly);
		Events.addEventHandler('read2', 'keypress', Events.integerOnly);
		Events.addEventHandler('select-pool', 'click', flowcell.onPoolSelected);
		Events.addEventHandler('wizard', 'click', flowcell.hidePoolSelection);

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', flowcell.submit);

		var url = '../FlowCell.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnusedPools';		
		Wizard.showLoadingAnimation('Loading pools...');
		Wizard.asyncJsonRequest(url, flowcell.initializeStep1);
	}


	flowcell.initializeStep1 = function(response)
	{
		flowcell.poolInfoLoaded(response);
		flowcell.presetOnChange();
		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	flowcell.poolInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var pools = response.pools;
		
		var poolsList = frm.pools;
		if (pools.length > 0)
		{
			for (var poolNo=0; poolNo < pools.length; poolNo++)
			{
				var pool = pools[poolNo];
				var name = pool.name + ' (' + Numbers.formatNumber(pool.molarity, 2, 'nM')  + ') - ';
				
				var numPlates = pool.libPlates.length;
				var title = '';

				if (numPlates <= 2)
				{
					for (var plateNo=0; plateNo < numPlates; plateNo++)
					{
						var libPlate = pool.libPlates[plateNo];
						if (plateNo > 0) name += ', ';
						name += libPlate.name;
					}
				}
				else
				{
					name += pool.libPlates[0].name + ' + ' + (numPlates-1) + ' more...';
					for (var plateNo=0; plateNo < numPlates; plateNo++)
					{
						var libPlate = pool.libPlates[plateNo];
						if (plateNo > 0) title += ', ';
						title += libPlate.name;
					}
				}
				
				var option = new Option(name, pool.id, false, poolNo < 1);
				option.title = title;
				option.pool = pool;
				
				poolsList[poolsList.length] = option;
			}
		}
	}

	flowcell.presetOnChange = function()
	{
		var frm = document.forms['reggie'];
		var preset = frm.preset.value;
		
		if (preset == PRESET_NEXTSEQ)
		{
			frm.num_lanes[0].disabled = true;
			frm.num_lanes[1].disabled = false;
			frm.num_lanes[2].disabled = true;
			frm.num_lanes[1].checked = true;
			for (var i = 0; i<frm.pools.length; i++)
			{
				frm.pools[i].selected = i < 1;
			}
			frm.read1.value = DEFAULT_READ_NEXTSEQ[0];
			frm.indexRead.value = DEFAULT_READ_NEXTSEQ[1];
			frm.read2.value = DEFAULT_READ_NEXTSEQ[2];
		}
		else
		{
			frm.num_lanes[0].disabled = false;
			frm.num_lanes[1].disabled = false;
			frm.num_lanes[2].disabled = false;
		}
		
		flowcell.poolsOnChange();
	}

	flowcell.poolsOnChange = function()
	{
		poolsAreValid = false;
		
		var frm = document.forms['reggie'];
		var preset = frm.preset.value;
		var poolsList = frm.pools;
		
		var numSelected = 0;
		var invalidPipeline = null;
		for (var i = 0; i < poolsList.length; i++)
		{
			if (poolsList[i].selected) 
			{
				var pool = poolsList[i].pool;
				if (pool.pipeline && pool.pipeline != 'RNAseq')
				{
					invalidPipeline = 'Pool ' + Strings.encodeTags(pool.name+' ('+pool.pipeline)+') is not intended for RNAseq pipeline.';
				}
				numSelected++;
			}
		}
		
		Doc.element('numSelected').innerHTML = numSelected + ' selected';
		
		if (invalidPipeline) 
		{
			Wizard.setInputStatus('pools', 'invalid', invalidPipeline);
			return;
		}
		if (numSelected == 0)
		{
			Wizard.setInputStatus('pools', 'invalid', 'Must select at least 1 pool.');
			return;
		}
		if (preset == PRESET_NEXTSEQ)
		{
			if (numSelected > 1)
			{
				Wizard.setInputStatus('pools', 'invalid', 'Can only use 1 pool with the NextSeq.');
				return;
			}
		}
		
		poolsAreValid = true;
		Wizard.setInputStatus('pools', 'valid');
	}
	
	
	flowcell.selectPools = function()
	{
		var frm = document.forms['reggie'];
		if (frm.pools.disabled) return;
		
		if (subtypePooledLibrary == null)
		{
			subtypePooledLibrary = Reggie.getSubtypeInfo('POOLED_LIBRARY');
		}
		if (annotationTypePipeline == null)
		{
			annotationTypePipeline = Reggie.getAnnotationTypeInfo('PIPELINE');
		}
		
		newPools = [];
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypePooledLibrary.id;
		url += '&tmpfilter:DATE:creationEvent.eventDate='+encodeURIComponent('<>');
		url += '&tmpfilter:STRING:'+encodeURIComponent('#')+annotationTypePipeline.id+'=RNAseq';
		Dialogs.selectItem('EXTRACT', 'pools', 1, url);
	}
	
	flowcell.setPoolCallback = function(event)
	{
		var frm = document.forms['reggie'];
		var name = event.detail.name;
		var id = event.detail.id;
		
		var poolsList = frm.pools;
		var isNew = true;
		for (var i = 0; i < poolsList.length; i++)
		{
			if (poolsList[i].value == id)
			{
				poolsList[i].selected = true;
				isNew = false;
			}
		}

		if (isNew)
		{
			newPools[newPools.length] = id;
		}
		
		if (event.detail.remaining == 0 && newPools.length > 0)
		{
			// Get more information about the selected library
			var url = '../FlowCell.servlet?ID='+App.getSessionId();
			url += '&cmd=GetPoolInfo&pools=' + newPools.join(',');
			Wizard.asyncJsonRequest(url, flowcell.manualPoolInfoLoaded);
		}
	}

	flowcell.manualPoolInfoLoaded = function(response)
	{
		flowcell.poolInfoLoaded(response);
		flowcell.poolsOnChange();
	}
	
	flowcell.validateStep1 = function(event)
	{
		if (!poolsAreValid) event.preventDefault();
	}

	
	flowcell.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];

		// Auto-select either 10µl or 20µl
		var maxMolarity = null;
		var minMolarity = null;
		var poolsList = frm.pools;
		for (var i = 0; i < poolsList.length; i++)
		{
			if (poolsList[i].selected) 
			{
				var pool = poolsList[i].pool;
				if (maxMolarity == null || pool.molarity > maxMolarity)
				{
					maxMolarity = pool.molarity;
				}
				if (minMolarity == null || pool.molarity < minMolarity)
				{
					minMolarity = pool.molarity;
				}
			}
		}

		if (Math.abs(maxMolarity - 1) < 0.25 && Math.abs(minMolarity - 1) < 0.25)
		{
			// Close to 1nM use 20µl instead of 10
			Forms.selectListOption(frm.volumeToUseFromPool, '20');
		}
		
		var numFlowCells = Forms.getCheckedRadio(frm.num_flowcells).value;
		var url = '../FlowCell.servlet?ID='+App.getSessionId();
		url += '&cmd=GetNextAutoGeneratedFlowCellNames&numNames='+numFlowCells;
		Wizard.showLoadingAnimation('Loading flow cell information...');
		Wizard.asyncJsonRequest(url, flowcell.flowCellsLoaded);
	}
	
	flowcell.flowCellsLoaded = function(response)
	{
		var frm = document.forms['reggie'];

		var names = response.names;
		var numFlowCells = names.length;
		var numLanes = Forms.getCheckedRadio(frm.num_lanes).value;

		// Initialize flowCell->lane->pool array
		for (var fcNo = 0; fcNo < numFlowCells; fcNo++)
		{
			var fc = {};
			fc.name = names[fcNo];
			fc.lanes = [];
			for (var laneNo = 0; laneNo < numLanes; laneNo++)
			{
				fc.lanes[laneNo] = {};
			}
			
			flowCells[fcNo] = fc;
			Doc.element('fc.'+fcNo).innerHTML = Strings.encodeTags(fc.name);
			Doc.element('comments.'+fcNo+'.name').innerHTML = Strings.encodeTags(fc.name);
		}
		
		// Hide unused flow cell
		if (numFlowCells == 1)
		{
			Doc.addClass(plate, 'hide-fc-1');
			Doc.hide('comments.1');
		}

		// Hide unused lanes
		for (var i = numLanes; i < 8; i++)
		{
			Doc.hide('lane.'+i);
		}

		// Generate popup menu for pool selection
		var selectPoolHtml = '';	
		for (var i = 0; i < frm.pools.length; i++)
		{
			if (frm.pools[i].selected)
			{
				var pool = frm.pools[i].pool;
				var index = selectedPools.length;
				selectedPools[index] = pool;

				selectPoolHtml += '<div class="menuitem enabled interactable" id="pool-'+pool.id+'" data-index="'+index+'">';
				selectPoolHtml += Strings.encodeTags(pool.name)+'</div>';
			}
		}
		// Add separator
		selectPoolHtml += '<div class="menuseparator"></div>';
		// Add option for empty lane
		selectPoolHtml += '<div class="menuitem enabled interactable" id="pool-none"><i>empty</i>'+'</div>';		
		Doc.element('select-pool-all').innerHTML = selectPoolHtml;

		// A single pool on all lanes
		if (selectedPools.length == 1)
		{
			var pool = selectedPools[0];
			for (var fcNo = 0; fcNo < numFlowCells; fcNo++)
			{
				for (var laneNo = 0; laneNo < numLanes; laneNo++)
				{
					flowCells[fcNo].lanes[laneNo] = {'pool': pool, 'defaultPool': pool};
				}
			}
		}
		
		flowcell.updateFlowCellsMatrix(true);
		
		Doc.show('gocancel');
		Doc.show('goregister');
		Wizard.setCurrentStep(2);
		
		Events.sendChangeEvent('read1');
		Events.sendChangeEvent('indexRead');
		Events.sendChangeEvent('read2');
		frm.read1.focus();

	}
	
	flowcell.readOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var target = event.currentTarget;
		
		Wizard.setInputStatus(target.id);

		var defaultValue = Data.int(target, 'default-value');
		var value = target.value;
		if (!defaultValue) Data.set(target, 'default-value', value); // Initalize the default value
		
		if (value == '')
		{
			readIsValid[target.id] = false;
			Wizard.setInputStatus(target.id, 'invalid', 'Missing');
			return;
		}
		
		if (!Numbers.isInteger(value))
		{
			readIsValid[target.id] = false;
			Wizard.setInputStatus(target.id, 'invalid', 'Not a number');
			return;
		}
		
		readIsValid[target.id] = true;
		if (defaultValue && parseInt(value) != defaultValue)
		{
			Wizard.setInputStatus(target.id, 'warning', 'Default: ' + defaultValue);
		}
		else
		{
			Wizard.setInputStatus(target.id, 'valid');
		}
	}

	flowcell.updateFlowCellsMatrix = function(attachEventHandlers)
	{
		Wizard.setInputStatus('flowCells');
		flowCellsAreValid = false;
		
		var frm = document.forms['reggie'];
		var numFlowCells = Forms.getCheckedRadio(frm.num_flowcells).value;
		var numLanes = Forms.getCheckedRadio(frm.num_lanes).value;
		var numEmptyLanes = 0;
		
		for (var fcNo = 0; fcNo < flowCells.length; fcNo++)
		{
			for (var laneNo = 0; laneNo < flowCells[fcNo].lanes.length; laneNo++)
			{
				var lane = flowCells[fcNo].lanes[laneNo];
				var pool = lane.pool;
				
				var name = pool ? Strings.encodeTags(pool.name) : '';
				if (pool && lane.defaultPool && pool != lane.defaultPool)
				{
					name = '<span class="nondefault">' + name + '</name>';
				}
				
				Doc.element('lane.'+laneNo+'.'+fcNo).innerHTML = name;
				if (attachEventHandlers) 
				{
					Events.addEventHandler('lane.'+laneNo+'.'+fcNo, 'click', flowcell.selectPool);
				}
				if (!pool) numEmptyLanes++;
			}
		}

		if (numEmptyLanes >= numFlowCells*numLanes)
		{
			Wizard.setInputStatus('flowCells', 'invalid', 'All lanes empty');
			return;
		}
		
		flowCellsAreValid = true;
		if (numEmptyLanes > 0)
		{
			Wizard.setInputStatus('flowCells', 'warning', numEmptyLanes + ' empty lanes');
		}
		else
		{
			Wizard.setInputStatus('flowCells', 'valid');
		}
	}

	
	flowcell.selectPool = function(event)
	{
		var frm = document.forms['reggie'];
		if (frm.preset.value == PRESET_NEXTSEQ) return;
		
		var target = event.currentTarget;
		currentFcNo = Data.int(target, 'fc-no');
		currentLaneNo = Data.int(target, 'lane-no');
		
		// Reset 'current' selection
		var menu = Doc.element('select-pool');
		var selectAll = Doc.element('select-pool-all');
		for (var i = 0; i <  selectAll.childNodes.length; i++)
		{
			Doc.removeClass(selectAll.childNodes[i], 'current');
			Doc.removeClass(selectAll.childNodes[i], 'default');
		}
		menu.style.display = 'block';
		
		var currentPool = flowCells[currentFcNo].lanes[currentLaneNo].pool;
		var defaultPool = flowCells[currentFcNo].lanes[currentLaneNo].defaultPool;
		if (currentPool) 
		{
			Doc.addClass('pool-'+currentPool.id, 'current');
		}
		if (defaultPool && defaultPool != currentPool)
		{
			Doc.addClass('pool-'+defaultPool.id, 'default');
		}
	
		var x = event.clientX;
		var halfHeight = Math.floor(selectAll.offsetHeight/2)
		var y = event.clientY+2; //-halfHeight;

		var scroll = 0;
		
		// Position the selection div
		selectAll.scrollTop = scroll;
		menu.style.left = (x)+'px';
		menu.style.top = (y)+'px';
		event.stopPropagation();
	}

	flowcell.onPoolSelected = function(event)
	{
		var optionId = event.target.id;
		var pool = null;
		var index = Data.int(event.target, 'index', -1);
		if (index >= 0)
		{
			pool = selectedPools[index];
		}
		flowCells[currentFcNo].lanes[currentLaneNo].pool = pool;
		flowcell.updateFlowCellsMatrix();
	}
	
	flowcell.hidePoolSelection = function()
	{
		Doc.hide('select-pool');
	}

	flowcell.validateStep2 = function(event)
	{
		var valid = flowCellsAreValid;
		valid &= readIsValid['read1'];
		valid &= readIsValid['indexRead'];
		valid &= readIsValid['read2'];
		
		if (!valid)
		{
			event.preventDefault();
		}	
	}
	
	flowcell.submit = function()
	{
		var frm = document.forms['reggie'];
		var volumeToUseFromPool = parseFloat(frm.volumeToUseFromPool.value);

		var submitInfo = {};
		submitInfo.flowCellType = frm.preset.value;
		submitInfo.read1 = parseInt(frm.read1.value);
		submitInfo.indexRead = parseInt(frm.indexRead.value);
		submitInfo.read2 = parseInt(frm.read2.value);
		submitInfo.flowCells = flowCells;
		
		// Calculate volume to use for each pool
		for (var fcNo = 0; fcNo < flowCells.length; fcNo++)
		{
			var flowCell = flowCells[fcNo];
			flowCell.comment = frm['comments.'+fcNo].value;
			for (var laneNo = 0; laneNo < flowCell.lanes.length; laneNo++)
			{
				var lane = flowCell.lanes[laneNo];
				lane.defaultPool = null; // Do not need to submit this information
				var pool = lane.pool;
				if (pool != null)
				{
					pool.count = pool.count ? pool.count+1 : 1;
				}
			}
		}
		
		for (var poolNo = 0; poolNo < selectedPools.length; poolNo++)
		{
			var pool = selectedPools[poolNo];
			if (pool != null)
			{
				pool.usedVolume = volumeToUseFromPool / pool.count;
			}
		}
	
		var url = '../FlowCell.servlet?ID='+App.getSessionId();
		url += '&cmd=CreateFlowCells';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, flowcell.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	flowcell.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return flowcell;
}();

Doc.onLoad(FlowCell.initPage);

